@@ -16,6 +16,8 @@ struct app_args_t
 	std::uint16_t m_port{ 8080 };
 	std::size_t m_pool_size{ 1 };
 
+	std::size_t m_max_parallel_connections{ 0u };
+
 	static app_args_t
 	parse( int argc, const char * argv[] )
 	{
@@ -35,6 +37,14 @@ struct app_args_t
 					( fmt::format(
 						"The size of a thread pool to run server (default: {})",
 						result.m_pool_size ) )
+			| Opt( result.m_max_parallel_connections, "max parallel connections" )
+					[ "-m" ][ "--max-parallel-connections" ]
+					( fmt::format(
+						"The max count of parallel connections. "
+						"Zero means that connection count limits is not used. "
+						"(default: {})",
+						result.m_max_parallel_connections )
+					)
 			| Help(result.m_help);
 
 		auto parse_result = cli.parse( Args(argc, argv) );
