@@ -421,6 +421,98 @@ class json_output_t
 		rapidjson::MemoryPoolAllocator<> & m_allocator;
 };
 
+//
+// const_map_key_t
+//
+/*!
+ * @brief A special indicator for the case when serialized value is a key
+ * in map-like container.
+ *
+ * Since v.0.2.11 json_dto makes distinction between keys and values
+ * in map-like containers. Values are serialized by calling write_json_value
+ * for a reference of type T (where T is a type of a value). But keys are
+ * now serialized by calling write_json_value for an instance of type
+ * const_map_key_t<T> (where T is a type of a key).
+ *
+ * It allows to override write_json_value for const_map_key_t<T> for
+ * the implementation of custom format for keys.
+ *
+ * @since v.0.2.11
+ */
+template< typename T >
+struct const_map_key_t
+{
+	const T & v;
+};
+
+/*!
+ * @brief Helper function for producing instances of const_map_key_t.
+ *
+ * @since v.0.2.11
+ */
+template< typename T >
+const_map_key_t<T>
+const_map_key( const T & v ) noexcept { return { v }; };
+
+//
+// mutable_map_key_t
+//
+/*!
+ * @brief A special indicator for the case when deserialized value is a key
+ * in map-like container.
+ *
+ * Since v.0.2.11 json_dto makes distinction between keys and values
+ * in map-like containers. Values are deserialized by calling read_json_value
+ * for a reference of type T (where T is a type of a value). But keys are
+ * now deserialized by calling read_json_value for an instance of type
+ * mutable_map_key_t<T> (where T is a type of a key).
+ *
+ * It allows to override mutable_json_value for mutable_map_key_t<T> for
+ * the implementation of custom format for keys.
+ *
+ * @since v.0.2.11
+ */
+template< typename T >
+struct mutable_map_key_t
+{
+	T & v;
+};
+
+/*!
+ * @brief Helper function for producing instances of mutable_map_key_t.
+ *
+ * @since v.0.2.11
+ */
+template< typename T >
+mutable_map_key_t<T>
+mutable_map_key( T & v ) noexcept { return { v }; };
+
+//
+// default_reader_writer_t
+//
+// NOTE: implementation is going below.
+/*!
+ * @brief The default implementation of Reader_Writer.
+ *
+ * This implementation simply calls read_json_value and write_json_value
+ * functions.
+ *
+ * @since v.0.2.10
+ */
+struct default_reader_writer_t
+{
+	template< typename Field_Type >
+	void
+	read( Field_Type & v, const rapidjson::Value & from ) const;
+
+	template< typename Field_Type >
+	void
+	write(
+		const Field_Type & v,
+		rapidjson::Value & to,
+		rapidjson::MemoryPoolAllocator<> & allocator ) const;
+};
+
 //
 // reader functions.
 //
@@ -544,6 +636,43 @@ write_json_value(
 	object.SetString( s.data(), static_cast< rapidjson::SizeType >( s.size() ), allocator );
 }
 
+// Since v.0.2.10.
+inline void
+write_json_value(
+	const rapidjson::Value::StringRefType & s,
+	rapidjson::Value & object,
+	rapidjson::MemoryPoolAllocator<> & allocator )
+{
+	// NOTE: there is no check for max_str_len as for std::string version
+	// because s.length has type rapidjson::SizeType.
+	object.SetString( s.s, s.length, allocator );
+}
+
+//
+// const- and mutable map keys
+//
+
+// Since v.0.2.11.
+template< typename T >
+void
+read_json_value(
+	mutable_map_key_t<T> key,
+	const rapidjson::Value & object )
+{
+	read_json_value( key.v, object );
+}
+
+// Since v.0.2.11.
+template< typename T >
+void
+write_json_value(
+	const_map_key_t<T> key,
+	rapidjson::Value & object,
+	rapidjson::MemoryPoolAllocator<> & allocator )
+{
+	write_json_value( key.v, object, allocator );
+}
+
 //
 // JSON
 //
@@ -567,96 +696,106 @@ write_json_value(
 //
 // std::optional
 //
-template< typename T >
+template< typename T, typename Reader_Writer = default_reader_writer_t >
 inline void
 read_json_value(
 	cpp17::optional<T> & v,
-	const rapidjson::Value & object );
+	const rapidjson::Value & object,
+	Reader_Writer reader_writer = Reader_Writer{} );
 
-template< typename T >
+template< typename T, typename Reader_Writer = default_reader_writer_t >
 inline void
 write_json_value(
 	const cpp17::optional<T> & v,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator );
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	Reader_Writer reader_writer = Reader_Writer{} );
 #endif
 
 //
 // ARRAY
 //
 
-template< typename T, typename A >
+template< typename T, typename A, typename Reader_Writer = default_reader_writer_t >
 void
 read_json_value(
 	std::vector< T, A > & vec,
-	const rapidjson::Value & object );
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
-template< typename T, typename A >
+template< typename T, typename A, typename Reader_Writer = default_reader_writer_t >
 void
 write_json_value(
 	const std::vector< T, A > & vec,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator );
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
 //
 // STL-like non-associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_like_sequence_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object );
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_like_sequence_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator );
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
 //
 // STL-set-like associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_set_like_associative_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object );
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_set_like_associative_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator );
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
 //
 // STL-map-like associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_map_like_associative_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object );
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer = default_reader_writer_t{} );
 
-template< typename C >
+template< typename C, typename Reader_Writer = default_reader_writer_t >
 std::enable_if_t<
 		details::meta::is_stl_map_like_associative_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator );
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer = Reader_Writer{} );
 
 //
 // nullable_t
@@ -666,11 +805,11 @@ write_json_value(
 template< typename Field_Type >
 struct nullable_t
 {
-	nullable_t()
+	nullable_t() noexcept
 		:	m_has_value{ false }
 	{}
 
-	nullable_t( std::nullptr_t )
+	nullable_t( std::nullptr_t ) noexcept
 		:	m_has_value{ false }
 	{}
 
@@ -707,12 +846,12 @@ struct nullable_t
 	}
 
 	bool
-	has_value() const
+	has_value() const noexcept
 	{
 		return m_has_value;
 	}
 
-	operator bool () const
+	operator bool () const noexcept
 	{
 		return has_value();
 	}
@@ -774,32 +913,32 @@ struct nullable_t
 	}
 
 	nullable_t &
-	operator = ( std::nullptr_t )
+	operator = ( std::nullptr_t ) noexcept
 	{
 		reset();
 		return *this;
 	}
 
 	const Field_Type*
-	operator -> () const
+	operator -> () const noexcept
 	{
 		return field_ptr();
 	}
 
 	Field_Type*
-	operator -> ()
+	operator -> () noexcept
 	{
 		return field_ptr();
 	}
 
 	const Field_Type &
-	operator * () const
+	operator * () const noexcept
 	{
 		return field_ref();
 	}
 
 	Field_Type&
-	operator * ()
+	operator * () noexcept
 	{
 		return field_ref();
 	}
@@ -826,7 +965,7 @@ struct nullable_t
 	}
 
 	void
-	reset()
+	reset() noexcept
 	{
 		if( has_value() )
 		{
@@ -840,25 +979,25 @@ struct nullable_t
 		bool m_has_value{ false };
 
 		Field_Type *
-		field_ptr()
+		field_ptr() noexcept
 		{
 			return reinterpret_cast< Field_Type * >( m_image_space );
 		}
 
 		const Field_Type *
-		field_ptr() const
+		field_ptr() const noexcept
 		{
 			return reinterpret_cast< const Field_Type * >( m_image_space );
 		}
 
 		Field_Type &
-		field_ref()
+		field_ref() noexcept
 		{
 			return *field_ptr();
 		}
 
 		const Field_Type &
-		field_ref() const
+		field_ref() const noexcept
 		{
 			return *field_ptr();
 		}
@@ -913,26 +1052,32 @@ write_json_value(
 // RW specializations for nullable_t< T >
 //
 
-template< typename Field_Type >
+template<
+	typename Field_Type,
+	typename Reader_Writer = default_reader_writer_t >
 void
 read_json_value(
 	nullable_t< Field_Type > & f,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	Reader_Writer reader_writer = Reader_Writer{} )
 {
 	Field_Type value;
-	read_json_value( value, object );
+	reader_writer.read( value, object );
 	f = std::move( value );
 }
 
-template< typename Field_Type >
+template<
+	typename Field_Type,
+	typename Reader_Writer = default_reader_writer_t >
 void
 write_json_value(
-	nullable_t< Field_Type > & f,
+	const nullable_t< Field_Type > & f,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	Reader_Writer reader_writer = Reader_Writer{} )
 {
 	if( f )
-		write_json_value( *f, object, allocator );
+		reader_writer.write( *f, object, allocator );
 	else
 		object.SetNull();
 }
@@ -941,26 +1086,28 @@ write_json_value(
 //
 // std::optional
 //
-template< typename T >
+template< typename T, typename Reader_Writer >
 inline void
 read_json_value(
 	cpp17::optional<T> & v,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	Reader_Writer reader_writer )
 {
 	T value_from_stream;
-	read_json_value( value_from_stream, object );
+	reader_writer.read( value_from_stream, object );
 	v = std::move(value_from_stream);
 }
 
-template< typename T >
+template< typename T, typename Reader_Writer >
 inline void
 write_json_value(
 	const cpp17::optional<T> & v,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	Reader_Writer reader_writer )
 {
 	if( v )
-		write_json_value( *v, object, allocator );
+		reader_writer.write( *v, object, allocator );
 }
 #endif
 
@@ -968,11 +1115,12 @@ write_json_value(
 // ARRAY
 //
 
-template< typename T, typename A  >
+template< typename T, typename A, typename Reader_Writer >
 void
 read_json_value(
 	std::vector< T, A > & vec,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer )
 {
 	if( object.IsArray() )
 	{
@@ -981,7 +1129,7 @@ read_json_value(
 		for( rapidjson::SizeType i = 0; i < object.Size(); ++i )
 		{
 			T v;
-			read_json_value( v, object[ i ] );
+			reader_writer.read( v, object[ i ] );
 			vec.push_back( std::move(v) );
 		}
 	}
@@ -1008,18 +1156,19 @@ struct std_vector_item_read_access_type<bool>
 
 } /* namespace details */
 
-template< typename T, typename A >
+template< typename T, typename A, typename Reader_Writer >
 void
 write_json_value(
 	const std::vector< T, A > & vec,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer )
 {
 	object.SetArray();
 	for( typename details::std_vector_item_read_access_type<T>::type v : vec )
 	{
 		rapidjson::Value o;
-		write_json_value( v, o, allocator );
+		reader_writer.write( v, o, allocator );
 		object.PushBack( o, allocator );
 	}
 }
@@ -1027,13 +1176,14 @@ write_json_value(
 //
 // STL-like non-associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_like_sequence_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer )
 {
 	if( object.IsArray() )
 	{
@@ -1043,7 +1193,7 @@ read_json_value(
 		for( rapidjson::SizeType i = 0; i < object.Size(); ++i )
 		{
 			typename C::value_type v;
-			read_json_value( v, object[ i ] );
+			reader_writer.read( v, object[ i ] );
 			filler.emplace_back( std::move(v) );
 		}
 	}
@@ -1051,20 +1201,21 @@ read_json_value(
 		throw ex_t{ "value is not an array" };
 }
 
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_like_sequence_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer )
 {
 	object.SetArray();
 	for( const auto & v : cnt )
 	{
 		rapidjson::Value o;
-		write_json_value( v, o, allocator );
+		reader_writer.write( v, o, allocator );
 		object.PushBack( o, allocator );
 	}
 }
@@ -1072,13 +1223,14 @@ write_json_value(
 //
 // STL-set-like associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_set_like_associative_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer )
 {
 	if( !object.IsArray() )
 		throw ex_t{ "value can't be deserialized into std::set-like container!" };
@@ -1087,7 +1239,7 @@ read_json_value(
 	for( rapidjson::SizeType i = 0; i < object.Size(); ++i )
 	{
 		typename C::value_type v;
-		read_json_value( v, object[ i ] );
+		reader_writer.read( v, object[ i ] );
 		cnt.emplace( std::move(v) );
 	}
 }
@@ -1100,18 +1252,20 @@ read_json_value(
  * But this check has performance penalty. So at v.0.2.8 there is no
  * such check.
  */
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_set_like_associative_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer )
 {
-	const auto write_item = [&object, &allocator]( const auto & v ) {
+	const auto write_item =
+			[&object, &allocator, &reader_writer]( const auto & v ) {
 				rapidjson::Value o;
-				write_json_value( v, o, allocator );
+				reader_writer.write( v, o, allocator );
 				object.PushBack( o, allocator );
 			};
 
@@ -1123,13 +1277,14 @@ write_json_value(
 //
 // STL-map-like associative containers.
 //
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_map_like_associative_container<C>::value,
 		void >
 read_json_value(
 	C & cnt,
-	const rapidjson::Value & object )
+	const rapidjson::Value & object,
+	const Reader_Writer & reader_writer )
 {
 	if( !object.IsObject() )
 		throw ex_t{ "value can't be deserialized into std::map-like container!" };
@@ -1140,8 +1295,11 @@ read_json_value(
 		typename C::key_type key;
 		typename C::mapped_type value;
 
-		read_json_value( key, it->name );
-		read_json_value( value, it->value );
+		// It is necessary to have mutable_key_ref as a lvalue to pass
+		// a non-const reference to it to read() method of the reader_writer.
+		auto mutable_key_ref = mutable_map_key(key);
+		reader_writer.read( mutable_key_ref, it->name );
+		reader_writer.read( value, it->value );
 
 		cnt.emplace( typename C::value_type{ std::move(key), std::move(value) } );
 	}
@@ -1155,21 +1313,26 @@ read_json_value(
  * But this check has performance penalty. So at v.0.2.8 there is no
  * such check.
  */
-template< typename C >
+template< typename C, typename Reader_Writer >
 std::enable_if_t<
 		details::meta::is_stl_map_like_associative_container<C>::value,
 		void >
 write_json_value(
 	const C & cnt,
 	rapidjson::Value & object,
-	rapidjson::MemoryPoolAllocator<> & allocator )
+	rapidjson::MemoryPoolAllocator<> & allocator,
+	const Reader_Writer & reader_writer )
 {
-	const auto write_item = [&object, &allocator]( const auto & kv ) {
+	const auto write_item =
+			[&object, &allocator, &reader_writer]( const auto & kv ) {
 				rapidjson::Value key;
 				rapidjson::Value value;
 
-				write_json_value( kv.first, key, allocator );
-				write_json_value( kv.second, value, allocator );
+				// It is necessary to have const_key_ref as a lvalue to pass a
+				// const reference to it to write() method of the reader_writer.
+				auto const_key_ref = const_map_key(kv.first);
+				reader_writer.write( const_key_ref, key, allocator );
+				reader_writer.write( kv.second, value, allocator );
 
 				object.AddMember( key, value, allocator );
 			};
@@ -1281,7 +1444,7 @@ struct mandatory_attr_t
 
 	template< typename Field_Type >
 	constexpr bool
-	is_default_value( Field_Type & ) const
+	is_default_value( Field_Type & ) const noexcept
 	{
 		return false;
 	}
@@ -1332,14 +1495,14 @@ struct optional_attr_null_t
 {
 	template< typename Field_Type >
 	void
-	on_field_not_defined( nullable_t< Field_Type > & f ) const
+	on_field_not_defined( nullable_t< Field_Type > & f ) const noexcept
 	{
 		f.reset();
 	}
 
 	template< typename Field_Type >
 	bool
-	is_default_value( nullable_t< Field_Type > & f ) const
+	is_default_value( nullable_t< Field_Type > & f ) const noexcept
 	{
 		return !f.has_value();
 	}
@@ -1354,12 +1517,12 @@ struct optional_nodefault_attr_t
 {
 	template< typename Field_Type >
 	constexpr void
-	on_field_not_defined( Field_Type & ) const
+	on_field_not_defined( Field_Type & ) const noexcept
 	{}
 
 	template< typename Field_Type >
 	constexpr bool
-	is_default_value( Field_Type & ) const
+	is_default_value( Field_Type & ) const noexcept
 	{
 		return false;
 	}
@@ -1372,7 +1535,7 @@ using string_ref_t = rapidjson::Value::StringRefType;
  * @since v.0.2.9
  */
 inline string_ref_t
-make_string_ref( const char * src, std::size_t length )
+make_string_ref( const char * src, std::size_t length ) noexcept
 {
 	return { src, static_cast<rapidjson::SizeType>(length) };
 }
@@ -1382,7 +1545,7 @@ make_string_ref( const char * src, std::size_t length )
  * @since v.0.2.9
  */
 inline string_ref_t
-make_string_ref( const std::string & src )
+make_string_ref( const std::string & src ) noexcept
 {
 	return make_string_ref( src.data(), src.size() );
 }
@@ -1392,7 +1555,7 @@ make_string_ref( const std::string & src )
  * @since v.0.2.9
  */
 inline string_ref_t
-make_string_ref( const char * src )
+make_string_ref( const char * src ) noexcept
 {
 	return make_string_ref( src, std::strlen(src) );
 }
@@ -1405,16 +1568,203 @@ struct empty_validator_t
 {
 	template< typename Field_Type >
 	constexpr void
-	operator () ( const Field_Type & ) const
+	operator () ( const Field_Type & ) const noexcept
 	{}
 };
 
+//
+// the implementation of default_reader_writer_t
+//
+
+template< typename Field_Type >
+void
+default_reader_writer_t::read(
+	Field_Type & v, const rapidjson::Value & from ) const
+{
+	read_json_value( v, from );
+}
+
+template< typename Field_Type >
+void
+default_reader_writer_t::write(
+	const Field_Type & v,
+	rapidjson::Value & to,
+	rapidjson::MemoryPoolAllocator<> & allocator ) const
+{
+	write_json_value( v, to, allocator );
+}
+
+/*!
+ * @brief A special proxy reader_writer that delegates actual
+ * formatting actions to the nested reader_writer.
+ *
+ * When a custom reader_writer is specified for a field it's applied
+ * to the field itself. For example:
+ * @code
+ * struct my_int_reader_writer {
+ * 	void read(int & v, ...) const {...}
+ * 	void write(const int & v, ...) const {...}
+ * };
+ *
+ * struct my_data {
+ * 	int field_;
+ * 	...
+ * 	template<typename Io> void json_io(Io & io) {
+ * 		io & json_dto::mandatory(my_int_reader_writer{}, "field", field_)
+ * 			...
+ * 			;
+ * 	}
+ * };
+ * @endcode
+ * In that case a reference to integer field `field_` will be passed to
+ * `my_int_reader_writer`. And this is intended behavior.
+ *
+ * But there could be cases when (de)serialized field is a container.
+ * For example, in that case we'll get a compiler error:
+ * @code
+ * struct my_complex_data {
+ * 	std::vector<int> field_;
+ * 	...
+ * 	template<typename Io> void json_io(Io & io) {
+ * 		io & json_dto::mandatory(my_int_reader_writer{}, "field", field_)
+ * 			...
+ * 			;
+ * 	}
+ * };
+ * @endcode
+ * It is because a reference to `vector<int>` will be passed to
+ * `my_int_reader_writer`, but `my_int_reader_writer` expects references
+ * to integers.
+ *
+ * It seems that this problem can be solved by extending `my_int_reader_writer`:
+ * @code
+ * struct my_int_reader_writer {
+ * 	void read(int & v, ...) const {...}
+ * 	void read(std::vector<int> & v, ...) const {...}
+ *
+ * 	void write(const int & v, ...) const {...}
+ * 	void write(const std::vector<int> & v, ...) const {...}
+ * };
+ * @endcode
+ * But this solution doesn't scale well because there are a plenty of
+ * container types. And it's not good to create an overload for every
+ * of them.
+ *
+ * So the template `apply_to_content_t` gives another solution:
+ * @code
+ * struct my_complex_data {
+ * 	std::vector<int> field_;
+ * 	...
+ * 	template<typename Io> void json_io(Io & io) {
+ * 		io & json_dto::mandatory(
+ * 				json_dto::apply_to_content_t<my_int_reader_writer>{},
+ * 				"field", field_)
+ * 			...
+ * 			;
+ * 	}
+ * };
+ * @endcode
+ *
+ * An instance of `apply_to_content_t` holds an instance of an actual
+ * reader_writer type and applies it to every item of container.
+ *
+ * @note
+ * The template `apply_to_content_t` should also be used will
+ * `nullable_t` and `std::optional` if an actual reader_writer has
+ * to be applied to the content of `nullable_t`/`std::optional`:
+ * @code
+ * struct my_data {
+ * 	std::optional<int> weight_;
+ * 	json_dto::nullable_t<int> priority_;
+ * 	...
+ * 	template<typename Io> void json_io(Io & io) {
+ * 		io & json_dto::optional(
+ * 				json_dto::apply_to_content_t<my_int_reader_writer>{},
+ * 				"weight", weight_, std::nullopt)
+ * 			& json_dto::mandatory(
+ * 				json_dto::apply_to_content_t<my_int_reader_writer>{},
+ * 				"priority", priority_)
+ * 		...
+ * 		;
+ * 	}
+ * }
+ * @endcode
+ *
+ * @note
+ * Sometimes `apply_to_content_t` should be nested:
+ * @code
+ * struct my_complex_data {
+ * 	json_dto::nullable_t< std::vector<int> > params_;
+ * 	...
+ * 	template<typename Io> void json_io(Io & io) {
+ * 		io & json_dto::mandatory(
+ * 				// The first occurence of apply_to_content_t is for nullable_t.
+ * 				json_dto::apply_to_content_t<
+ * 					// The second occurence of apply_to_content_t is for std::vector.
+ * 					json_dto::apply_to_content_t<
+ * 						// This is for the content of std::vector.
+ * 						my_int_reader_writer
+ * 					>
+ * 				>{},
+ * 				"params", params_)
+ * 			...
+ * 			;
+ * 	}
+ * };
+ * @endcode
+ *
+ * @since v.0.2.11
+ */
+template< typename Item_Reader_Writer >
+struct apply_to_content_t
+{
+	Item_Reader_Writer m_reader_writer;
+
+	apply_to_content_t() = default;
+
+	apply_to_content_t( const Item_Reader_Writer & initial )
+		:	m_reader_writer{ initial }
+	{}
+
+	apply_to_content_t( Item_Reader_Writer && initial )
+		:	m_reader_writer{ std::move(initial) }
+	{}
+
+	template< typename Field_Type >
+	void
+	read(
+		Field_Type & v, const rapidjson::Value & from ) const
+	{
+		read_json_value( v, from, m_reader_writer );
+	}
+
+	template< typename Field_Type >
+	void
+	write(
+		Field_Type & v,
+		rapidjson::Value & to,
+		rapidjson::MemoryPoolAllocator<> & allocator ) const
+	{
+		write_json_value( v, to, allocator, m_reader_writer );
+	}
+};
+
 //
 // binder_t
 //
 
 //! JSON IO binder_t for a field.
+/*!
+ * @note
+ * This template was extended in v.0.2.10 by a new template parameter
+ * Reader_Writer. This parameter specifies a type of object with
+ * two const methods `read` and `write`. Since v.0.2.10 those methods
+ * are used for reading and writting a value of Field_Type from/to
+ * rapidjson::Value. See default_reader_writer_t for an example
+ * of Reader_Writer.
+ */
 template<
+		typename Reader_Writer,
 		typename Field_Type,
 		typename Manopt_Policy,
 		typename Validator >
@@ -1422,11 +1772,13 @@ class binder_t
 {
 	public:
 		binder_t(
+			Reader_Writer && reader_writer,
 			string_ref_t field_name,
 			Field_Type & field,
 			Manopt_Policy && manopt_policy,
 			Validator && validator )
-			:	m_field_name{ field_name }
+			:	m_reader_writer{ std::move(reader_writer) }
+			,	m_field_name{ field_name }
 			,	m_field{ field }
 			,	m_manopt_policy{ std::move( manopt_policy ) }
 			,	m_validator{ std::move( validator ) }
@@ -1488,7 +1840,7 @@ class binder_t
 
 				if( !value.IsNull() )
 				{
-					read_json_value( m_field, value );
+					m_reader_writer.read( m_field, value );
 				}
 				else
 				{
@@ -1514,7 +1866,7 @@ class binder_t
 			{
 				rapidjson::Value value;
 
-				write_json_value( m_field, value, allocator );
+				m_reader_writer.write( m_field, value, allocator );
 
 				object.AddMember(
 					m_field_name,
@@ -1523,6 +1875,7 @@ class binder_t
 			}
 		}
 
+		Reader_Writer m_reader_writer;
 		string_ref_t m_field_name;
 		Field_Type & m_field;
 		Manopt_Policy m_manopt_policy;
@@ -1543,9 +1896,40 @@ mandatory(
 	Field_Type & field,
 	Validator validator = Validator{} )
 {
-	using binder_type_t = binder_t< Field_Type, mandatory_attr_t, Validator >;
-	return
-		binder_type_t{
+	using binder_type_t = binder_t<
+			default_reader_writer_t,
+			Field_Type,
+			mandatory_attr_t,
+			Validator >;
+
+	return binder_type_t{
+			default_reader_writer_t{},
+			field_name,
+			field,
+			mandatory_attr_t{},
+			std::move( validator ) };
+}
+
+//! Create bind for a mandatory JSON field with validator.
+template<
+		typename Reader_Writer,
+		typename Field_Type,
+		typename Validator = empty_validator_t >
+auto
+mandatory(
+	Reader_Writer reader_writer,
+	string_ref_t field_name,
+	Field_Type & field,
+	Validator validator = Validator{} )
+{
+	using binder_type_t = binder_t<
+			Reader_Writer,
+			Field_Type,
+			mandatory_attr_t,
+			Validator >;
+
+	return binder_type_t{
+			std::move(reader_writer),
 			field_name,
 			field,
 			mandatory_attr_t{},
@@ -1569,10 +1953,43 @@ optional(
 	Validator validator = Validator{} )
 {
 	using opt_attr_t = optional_attr_t< Field_Default_Value_Type >;
-	using binder_type_t = binder_t< Field_Type, opt_attr_t, Validator >;
+	using binder_type_t = binder_t<
+			default_reader_writer_t,
+			Field_Type,
+			opt_attr_t,
+			Validator >;
 
-	return
-		binder_type_t{
+	return binder_type_t{
+			default_reader_writer_t{},
+			field_name,
+			field,
+			opt_attr_t{ std::move( default_value ) },
+			std::move( validator ) };
+}
+
+//! Create bind for an optional JSON field with default value and validator.
+template<
+		typename Reader_Writer,
+		typename Field_Type,
+		typename Field_Default_Value_Type,
+		typename Validator = empty_validator_t >
+auto
+optional(
+	Reader_Writer reader_writer,
+	string_ref_t field_name,
+	Field_Type & field,
+	Field_Default_Value_Type default_value,
+	Validator validator = Validator{} )
+{
+	using opt_attr_t = optional_attr_t< Field_Default_Value_Type >;
+	using binder_type_t = binder_t<
+			Reader_Writer,
+			Field_Type,
+			opt_attr_t,
+			Validator >;
+
+	return binder_type_t{
+			std::move(reader_writer),
 			field_name,
 			field,
 			opt_attr_t{ std::move( default_value ) },
@@ -1593,10 +2010,44 @@ optional_null(
 	Field_Type & field,
 	Validator validator = Validator{} )
 {
-	using binder_type_t = binder_t< Field_Type, optional_attr_null_t, Validator >;
-	return
-		binder_type_t{
-			field_name, field, optional_attr_null_t{}, std::move( validator ) };
+	using binder_type_t = binder_t<
+			default_reader_writer_t,
+			Field_Type,
+			optional_attr_null_t,
+			Validator >;
+
+	return binder_type_t{
+			default_reader_writer_t{},
+			field_name,
+			field,
+			optional_attr_null_t{},
+			std::move( validator ) };
+}
+
+//! Create bind for an optional JSON field with null default value .
+template<
+		typename Reader_Writer,
+		typename Field_Type,
+		typename Validator = empty_validator_t >
+auto
+optional_null(
+	Reader_Writer reader_writer,
+	string_ref_t field_name,
+	Field_Type & field,
+	Validator validator = Validator{} )
+{
+	using binder_type_t = binder_t<
+			Reader_Writer,
+			Field_Type,
+			optional_attr_null_t,
+			Validator >;
+
+	return binder_type_t{
+			std::move(reader_writer),
+			field_name,
+			field,
+			optional_attr_null_t{},
+			std::move( validator ) };
 }
 
 //
@@ -1617,6 +2068,26 @@ optional(
 	return optional_null( field_name, field, std::move( validator ) );
 }
 
+//! Create bind for an optional JSON field with null default value.
+template<
+		typename Reader_Writer,
+		typename Field_Type,
+		typename Validator = empty_validator_t >
+auto
+optional(
+	Reader_Writer reader_writer,
+	string_ref_t field_name,
+	Field_Type & field,
+	std::nullptr_t,
+	Validator validator = Validator{} )
+{
+	return optional_null(
+			std::move(reader_writer),
+			field_name,
+			field,
+			std::move( validator ) );
+}
+
 //
 // optional_no_default
 //
@@ -1631,9 +2102,44 @@ optional_no_default(
 	Field_Type & field,
 	Validator validator = Validator{} )
 {
-	using binder_type_t = binder_t< Field_Type, optional_nodefault_attr_t, Validator >;
+	using binder_type_t = binder_t<
+			default_reader_writer_t,
+			Field_Type,
+			optional_nodefault_attr_t,
+			Validator >;
+
 	return binder_type_t{
-			field_name, field, optional_nodefault_attr_t{}, std::move( validator ) };
+			default_reader_writer_t{},
+			field_name,
+			field,
+			optional_nodefault_attr_t{},
+			std::move( validator ) };
+}
+
+//! Create bind for an optional JSON field without default value.
+template<
+		typename Reader_Writer,
+		typename Field_Type,
+		typename Validator = empty_validator_t >
+auto
+optional_no_default(
+	Reader_Writer reader_writer,
+	string_ref_t field_name,
+	Field_Type & field,
+	Validator validator = Validator{} )
+{
+	using binder_type_t = binder_t<
+			Reader_Writer,
+			Field_Type,
+			optional_nodefault_attr_t,
+			Validator >;
+
+	return binder_type_t{
+			std::move(reader_writer),
+			field_name,
+			field,
+			optional_nodefault_attr_t{},
+			std::move( validator ) };
 }
 
 template< typename Dto >
@@ -1668,9 +2174,11 @@ to_json( const Dto & dto )
 
 	rapidjson::StringBuffer buffer;
 	rapidjson::Writer< rapidjson::StringBuffer > writer( buffer );
-	output_doc.Accept( writer );
+	const bool result = output_doc.Accept( writer );
+	if( !result )
+		throw ex_t{ "to_json: output_doc.Accept(writer) returns false" };
 
-	return buffer.GetString();
+	return { buffer.GetString(), buffer.GetSize() };
 }
 
 inline void
