@@ -2,6 +2,8 @@
 	restinio
 */
 
+#define NOMINMAX
+
 #include <catch2/catch.hpp>
 
 #include <fmt/format.h>
@@ -622,20 +624,34 @@ TEST_CASE( "just_result (string case)", "[just_result]" )
 
 	auto parser = produce<std::string>(
 			alternatives(
-				caseless_symbol_p('f') >> just_result("Female"s),
-				caseless_symbol_p('m') >> just_result("Male"s)
+				caseless_exact_p("Fem") >> just_result("Female"s),
+				caseless_exact_p("Mal") >> just_result("Male"s)
 			)
 		);
 
 	{
-		const auto result = try_parse( "f", parser );
+		const auto result = try_parse( "fem", parser );
 
 		REQUIRE( result );
 		REQUIRE( "Female"s == *result );
 	}
 
 	{
-		const auto result = try_parse( "m", parser );
+		const auto result = try_parse( "FEM", parser );
+
+		REQUIRE( result );
+		REQUIRE( "Female"s == *result );
+	}
+
+	{
+		const auto result = try_parse( "mal", parser );
+
+		REQUIRE( result );
+		REQUIRE( "Male"s == *result );
+	}
+
+	{
+		const auto result = try_parse( "MAL", parser );
 
 		REQUIRE( result );
 		REQUIRE( "Male"s == *result );
@@ -671,6 +687,43 @@ TEST_CASE( "convert", "[convert_transformer]" )
 	}
 }
 
+TEST_CASE( "convert with failures", "[convert_transformer]" )
+{
+	using namespace restinio::easy_parser;
+	using namespace std::string_literals;
+
+	const auto parser = produce<std::string>(
+			any_symbol_p()
+				>> convert([](char c) ->
+						restinio::expected_t< std::string, error_reason_t >
+					{
+						if( 'f' == c || 'm' == c )
+							return std::string(1u, c);
+						else
+							return restinio::make_unexpected(
+									error_reason_t::unexpected_character );
+					})
+				>> as_result()
+		);
+
+	{
+		const auto r = try_parse( "f", parser );
+		REQUIRE( r );
+		REQUIRE( "f"s == *r );
+	}
+
+	{
+		const auto r = try_parse( "m", parser );
+		REQUIRE( r );
+		REQUIRE( "m"s == *r );
+	}
+
+	{
+		const auto r = try_parse( "z", parser );
+		REQUIRE( !r );
+	}
+}
+
 TEST_CASE( "convert generic case", "[convert_transformer]" )
 {
 	using namespace restinio::easy_parser;
@@ -1079,6 +1132,67 @@ TEST_CASE( "alternatives, as_result and variant",
 	}
 }
 
+TEST_CASE( "force_only_this_alternative",
+		"[alternatives][force_only_this_alternative]" )
+{
+	using namespace restinio::http_field_parsers;
+
+	struct increment_t { int m_v; };
+	struct decrement_t { int m_v; };
+
+	using inc_or_dec_t = restinio::variant_t<increment_t, decrement_t>;
+
+	const auto try_parse = []( restinio::string_view_t what ) {
+		return restinio::easy_parser::try_parse( what,
+				produce<inc_or_dec_t>(
+					alternatives(
+						produce<increment_t>(
+							exact("increment"),
+							force_only_this_alternative(
+								symbol('='),
+								decimal_number_p<int>() >> &increment_t::m_v
+							)
+						) >> as_result(),
+						produce<decrement_t>(
+							token_p() >> skip(),
+							force_only_this_alternative(
+								symbol(' '),
+								decimal_number_p<int>() >> &decrement_t::m_v
+							)
+						) >> as_result()
+					)
+				) );
+	};
+
+	{
+		const auto result = try_parse( "increment=234" );
+
+		REQUIRE( result );
+		REQUIRE( 0 == result->index() );
+		REQUIRE( 234 == restinio::get<increment_t>(*result).m_v );
+	}
+
+	{
+		const auto result = try_parse( "increment 234" );
+
+		REQUIRE( !result );
+	}
+
+	{
+		const auto result = try_parse( "increment=2a4" );
+
+		REQUIRE( !result );
+	}
+
+	{
+		const auto result = try_parse( "decrement 234" );
+
+		REQUIRE( result );
+		REQUIRE( 1 == result->index() );
+		REQUIRE( 234 == restinio::get<decrement_t>(*result).m_v );
+	}
+}
+
 TEST_CASE( "token", "[token]" )
 {
 	using namespace restinio::http_field_parsers;
@@ -2378,3 +2492,59 @@ TEST_CASE( "comment producer", "[comment_producer]" )
 	}
 }
 
+TEST_CASE( "IPv4 parser", "[non_negative_decimal_number_p]" )
+{
+	namespace ep = restinio::easy_parser;
+
+	const auto one_group = ep::non_negative_decimal_number_p<std::uint8_t>();
+
+	const auto rule = ep::produce<std::uint32_t>(
+			ep::produce< std::array<std::uint8_t, 4> >(
+				ep::repeat(3u, 3u,
+					one_group >> ep::to_container(),
+					ep::symbol('.')),
+				one_group >> ep::to_container()
+			)
+			>> ep::convert( [](const auto & arr) {
+					std::uint32_t result{};
+					for(const auto o : arr) {
+						result <<= 8;
+						result |= o;
+					}
+					return result;
+				} )
+			>> ep::as_result()
+		);
+
+	{
+		const auto result = ep::try_parse( "", rule );
+		REQUIRE( !result );
+	}
+
+	{
+		const auto result = ep::try_parse( "127.0.0.1", rule );
+		REQUIRE( result );
+		REQUIRE( 0x7F000001u == *result );
+	}
+
+	{
+		const auto result = ep::try_parse( "192.168.1.1", rule );
+
+		REQUIRE( result );
+		REQUIRE( 0xC0A80101u == *result );
+	}
+
+	{
+		const auto result = ep::try_parse( "192 168.1.1", rule );
+
+		REQUIRE( !result );
+	}
+
+
+	{
+		const auto result = ep::try_parse( "192.168.1.1.", rule );
+
+		REQUIRE( !result );
+	}
+
+}
